@@ -1,6 +1,6 @@
 /* gcompris - paratrooper.c
  *
- * Time-stamp: <2001/07/11 01:53:46 bruno>
+ * Time-stamp: <2001/12/03 23:35:25 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -41,9 +41,6 @@ static GnomeCanvasItem *planeitem = NULL;
 static gint plane_x, plane_y;
 static gint planespeed_x, planespeed_y, windspeed;
 
-/* These are the index for managing the game rule */
-static gint plane_target, plane_last_target;
-
 typedef struct {
   gint number;
   GnomeCanvasItem *rootitem;
@@ -76,12 +73,13 @@ static  ParatrooperItem	     paratrooperItem;
 /* default gnome pixmap directory in which this game tales the icon */
 static char *pixmapsdir = "gcompris/misc/";
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board ();
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
-gint key_press(guint keyval);
+static void	 start_board (GcomprisBoard *agcomprisBoard);
+static void	 pause_board (gboolean pause);
+static void	 end_board (void);
+static gboolean	 is_our_board (GcomprisBoard *gcomprisBoard);
+static void	 set_level (guint level);
+gint		 key_press(guint keyval);
+static void	 help(void);
 
 static GnomeCanvasItem *paratrooper_create_cloud(GnomeCanvasGroup *parent);
 static gint paratrooper_drop_clouds (GtkWidget *widget, gpointer data);
@@ -114,6 +112,8 @@ BoardPlugin menu_bp =
    key_press,
    NULL,
    set_level,
+   help,
+   NULL
 };
 
 /*
@@ -182,6 +182,7 @@ static void start_board (GcomprisBoard *agcomprisBoard)
       /* set initial values for this level */
       gcomprisBoard->level = 1;
       gcomprisBoard->maxlevel = 6;
+      gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_HELP);
 
       /* Init of paratrooper struct */
       paratrooperItem.rootitem = NULL;
@@ -317,6 +318,10 @@ gint key_press(guint keyval)
       break;
     case TUX_LANDED:
       gcomprisBoard->level++;
+      if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+	board_finished(BOARD_FINISHED_RANDOM);
+	return TRUE;
+      }
       paratrooper_next_level();
       break;
     case TUX_CRASHED:
@@ -325,7 +330,7 @@ gint key_press(guint keyval)
       paratrooperItem.status	= TUX_INPLANE;
       paratrooperItem.x		= 0;
       paratrooperItem.y		= 60;
-      paratrooperItem.speed		= (1 + rand()%gcomprisBoard->level)*3;
+      paratrooperItem.speed	= 3;
       gnome_canvas_item_hide(paratrooperItem.paratrooper);
       gnome_canvas_item_show(planeitem);
       break;
@@ -352,6 +357,25 @@ is_our_board (GcomprisBoard *gcomprisBoard)
   return FALSE;
 }
 
+static void
+help ()
+{
+  if(gcomprisBoard!=NULL)
+    {
+      pause_board(TRUE);
+      gcompris_help_start(N_("The paratrooper game"), 
+			  N_("\
+Goal:\n\
+In this game, the children will try to make Tux the paratrooper lands safely on the\n\
+fishing boat. He/She needs to take care of the wind side and speed.\n\
+\n\
+Usage:\n\
+Click on any keyboard key to let Tux jump. Another keypress will open the
+'parachute'.\n
+"));
+			  
+    }
+}
 
 /*-------------------------------------------------------------------------------*/
 /*-------------------------------------------------------------------------------*/
@@ -369,17 +393,14 @@ static void paratrooper_next_level()
   paratrooper_destroy_all_items();
 
   /* Try the next level */
-  speed=10+(40/(gcomprisBoard->level));
-  gcomprisBoard->number_of_sublevel=1;
+  speed=100+(30/(gcomprisBoard->level));
+  gcomprisBoard->number_of_sublevel=0;
   /* Make the images tend to 0.5 ratio */
-  imageZoom=0.5+(0.5/(gcomprisBoard->level));
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  imageZoom=0.3+(0.4/(gcomprisBoard->level));
 
   /* Setup and Display the plane */
   planespeed_y = 0;
-  planespeed_x = 1 + gcomprisBoard->level;
+  planespeed_x = 4 + gcomprisBoard->level;
   str = g_strdup_printf("%s%s", pixmapsdir, "tuxplane.png");
   pixmap = gcompris_load_pixmap(str);
   plane_x = 0;
@@ -391,10 +412,12 @@ static void paratrooper_next_level()
 				     "y", (double) plane_y,
 				     "width", (double) gdk_pixbuf_get_width(pixmap)*imageZoom,
 				     "height", (double) gdk_pixbuf_get_height(pixmap)*imageZoom,
+				     "width_set", TRUE, 
+				     "height_set", TRUE,
 				     NULL);
   gdk_pixbuf_unref(pixmap);
  
-  windspeed = (1 + rand()%gcomprisBoard->level);
+  windspeed = (3 + rand()%gcomprisBoard->level);
   if(rand()%2==0)
     windspeed *= -1;
 
@@ -402,7 +425,7 @@ static void paratrooper_next_level()
   if(drop_cloud_id)
     gtk_timeout_remove (drop_cloud_id);
 
-  drop_cloud_id = gtk_timeout_add (100,
+  drop_cloud_id = gtk_timeout_add (200,
 				   (GtkFunction) paratrooper_drop_clouds, NULL);
 
   /* Display the target */
@@ -410,7 +433,7 @@ static void paratrooper_next_level()
   str = g_strdup_printf("%s%s", pixmapsdir, "fishingboat.png");
   pixmap = gcompris_load_pixmap(str);
   boat_x = 350;
-  boat_y = 500;
+  boat_y = gcomprisBoard->height-100;
   boat_landarea_y = boat_y+20;
   boat_length = gdk_pixbuf_get_width(pixmap)*imageZoom;
   boatitem = gnome_canvas_item_new (gnome_canvas_root(gcomprisBoard->canvas),
@@ -419,7 +442,9 @@ static void paratrooper_next_level()
 				    "x", (double) boat_x,
 				    "y", (double) boat_y,
 				    "width", (double) gdk_pixbuf_get_width(pixmap)*imageZoom,
-				    "height", (double) gdk_pixbuf_get_height(pixmap)*imageZoom,
+				    "height", (double) gdk_pixbuf_get_height(pixmap),
+				    "width_set", TRUE, 
+				    "height_set", TRUE,
 				    NULL);
   gdk_pixbuf_unref(pixmap);
 
@@ -446,7 +471,7 @@ static void paratrooper_next_level()
   paratrooperItem.status	= TUX_INPLANE;
   paratrooperItem.x		= 0;
   paratrooperItem.y		= 60;
-  paratrooperItem.speed		= (1 + rand()%gcomprisBoard->level)*3;
+  paratrooperItem.speed		= 3;
 
   paratrooperItem.rootitem = \
     gnome_canvas_item_new (gnome_canvas_root(gcomprisBoard->canvas),
@@ -478,7 +503,7 @@ static void paratrooper_next_level()
 						     gnome_canvas_pixbuf_get_type (),
 						     "pixbuf", pixmap, 
 						     "x", (double) -gdk_pixbuf_get_width(pixmap)/2,
-						     "y", (double) -(gdk_pixbuf_get_height(pixmap)/2)-70,
+						     "y", (double) -(gdk_pixbuf_get_height(pixmap)/2)-60,
 						     "width", (double) gdk_pixbuf_get_width(pixmap),
 						     "height", (double) gdk_pixbuf_get_height(pixmap),
 						     NULL);
@@ -488,12 +513,6 @@ static void paratrooper_next_level()
   /* The sea is always on top */
   gnome_canvas_item_raise_to_top(seaitem);
 
-  /* Game rules */
-  plane_target = 1;
-  plane_last_target = gcomprisBoard->level*10;
-  gcompris_bar_set_maxtimer(plane_last_target);
-  gcompris_bar_set_timer(plane_target-1);
-
   g_free (str);
 
 }
@@ -541,7 +560,7 @@ static void paratrooper_move_cloud(CloudItem *clouditem)
       item2del_list = g_list_append (item2del_list, clouditem);
 
       /* Drop a new cloud */
-      drop_cloud_id = gtk_timeout_add (100,
+      drop_cloud_id = gtk_timeout_add (200,
 				       (GtkFunction) paratrooper_drop_clouds, NULL);
     }
 
@@ -632,6 +651,7 @@ static gint paratrooper_move_items (GtkWidget *widget, gpointer data)
  */
 static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
 {
+  gint offset;
 
   /* Manage the wrapping */
   if(paratrooperItem.x<0) {
@@ -643,34 +663,37 @@ static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
     gnome_canvas_item_move(paratrooperItem.rootitem, -gcomprisBoard->width, 0);
   }
 
-  gnome_canvas_item_move(paratrooperItem.rootitem, windspeed, paratrooperItem.speed);
+  offset = ( windspeed<0 ? -1 : 1 );
+  gnome_canvas_item_move(paratrooperItem.rootitem, offset, paratrooperItem.speed);
   paratrooperItem.y += paratrooperItem.speed;
-  paratrooperItem.x += windspeed;
+  paratrooperItem.x += offset;
 
   /* Check we reached the target */
   if(paratrooperItem.y>boat_landarea_y)
     {
-      if(paratrooperItem.x>boat_x && paratrooperItem.x<boat_x+boat_length+20 &&
+      if(paratrooperItem.x>boat_x && paratrooperItem.x<boat_x+boat_length &&
 	 paratrooperItem.status==TUX_FLYING)
 	{
 	  gnome_canvas_item_hide(paratrooperItem.parachute);
 	  paratrooperItem.status = TUX_LANDED;
+	  gcompris_play_sound (SOUNDLISTFILE, "bonus");
 	}
       else
 	{
 	  if(paratrooperItem.y<gcomprisBoard->height-20)
-	    drop_tux_id = gtk_timeout_add (paratrooperItem.speed*10,
+	    drop_tux_id = gtk_timeout_add (150,
 					   (GtkFunction) paratrooper_move_tux, NULL);
 	  else
 	    {
 	      gnome_canvas_item_hide(paratrooperItem.parachute);
 	      paratrooperItem.status = TUX_CRASHED;
+	      gcompris_play_sound (SOUNDLISTFILE, "crash");
 	    }
 	}
     }
   else
     {
-      drop_tux_id = gtk_timeout_add (paratrooperItem.speed*10,
+      drop_tux_id = gtk_timeout_add (150,
 				     (GtkFunction) paratrooper_move_tux, NULL);
     }
 
